;;;
;;; circuit-tests.lisp
;;;
;;; Regression tests for CIRCUIT-SIM
;;;
;;; $Revision: 1.1 $
;;;
;;; $Log: circuit-tests.lisp,v $
;;; Revision 1.1  2007/02/11 19:29:07  jao
;;; Initial revision
;;;
;;;
;;;

(defpackage "CIRCUIT-TESTS"
  (:use "COMMON-LISP" "CIRCUIT" "RT"))

(in-package "CIRCUIT-TESTS")

(deftest logical-true-1 (circuit:logical-true t) t)
(deftest logical-true-2 (circuit:logical-true 1) t)
(deftest logical-true-3 (circuit:logical-true 0) nil)
(deftest logical-true-4 (circuit:logical-true nil) nil)
(deftest logical-true-5 (circuit:logical-true :invalid) nil)
(deftest logical-true-6 (circuit:logical-true 3) nil)

(deftest logical-false-1 (circuit:logical-false t) nil)
(deftest logical-false-2 (circuit:logical-false 1) nil)
(deftest logical-false-3 (circuit:logical-false 0) t)
(deftest logical-false-4 (circuit:logical-false nil) t)
(deftest logical-false-5 (circuit:logical-false :invalid) nil)
(deftest logical-false-6 (circuit:logical-false 3) nil)

(deftest logical-valid-1 (circuit:logical-valid t) t)
(deftest logical-valid-2 (circuit:logical-valid 1) t)
(deftest logical-valid-3 (circuit:logical-valid 0) t)
(deftest logical-valid-4 (circuit:logical-valid nil) t)
(deftest logical-valid-5 (circuit:logical-valid :invalid) nil)
(deftest logical-valid-6 (circuit:logical-valid 3) nil)

(deftest logical-eqv-1 (circuit:logical-eqv t t) t)
(deftest logical-eqv-2 (circuit:logical-eqv t 1) t)
(deftest logical-eqv-3 (circuit:logical-eqv 1 t) t)
(deftest logical-eqv-4 (circuit:logical-eqv nil t) nil)
(deftest logical-eqv-5 (circuit:logical-eqv nil nil) t)
(deftest logical-eqv-6 (circuit:logical-eqv nil 0) t)
(deftest logical-eqv-7 (circuit:logical-eqv nil :invalid) nil)
(deftest logical-eqv-8 (circuit:logical-eqv t :invalid) nil)
(deftest logical-eqv-9 (circuit:logical-eqv :invalid :invalid) nil)

(deftest logical-and-1 (circuit:logical-and 0 0) nil)
(deftest logical-and-2 (circuit:logical-and 0 1) nil)
(deftest logical-and-3 (circuit:logical-and 1 0) nil)
(deftest logical-and-4 (circuit:logical-and 1 1) t)
(deftest logical-and-5 (circuit:logical-and 1 1 t :invalid) :invalid)
(deftest logical-and-6 (circuit:logical-and 1 1 t :invalid nil) nil)

(defun sample-times (wire time-list)
  "Calls propagate and returns a list of the values of WIRE 
   at the times listed in TIME-LIST (checking them in time-order, 
   not list-order)"
  (let (list)
    (mapcar (lambda (time) 
	      (circuit::at-time-fun time (lambda () 
					   (push (get-signal wire) list))))
	    time-list)
    (propagate)
    (nreverse list)))
						    
(deftest test-inverter
    (let ((d1 (make-wire))
	  (out (make-wire)))
      
      (setf *agenda* (make-agenda 0))
      (inverter d1 out 5)
      
      (at-time 0 (set-signal d1 0))
      (at-time 10 (set-signal d1 1))
      (at-time 20 (set-signal d1 :invalid))
      
      (sample-times out '(4 6 14 16 24 26)))
  (0 1 1 0 0 :invalid))

(deftest test-nand-gate
    (let ((d1 (make-wire))
	  (d2 (make-wire))
	  (out (make-wire)))
    
      (setf *agenda* (make-agenda 0))
      (nand-gate d1 d2 out 10)
      (at-time 0 
	(set-signal d1 0)
	(set-signal d2 0))
      (at-time 30 (set-signal d1 1))
      (at-time 50 (set-signal d2 1))
      (at-time 70 (set-signal d1 0))

      (sample-times out '(5 9 11 39 41 59 61 79 81)))
  (0 0 1 1 1 1 0 0 1))

(deftest sicp-example 
    (let ((inv-delay 2)
	  (and-delay 3)
	  (or-delay 5)
	  (in1 (make-wire))
	  (in2 (make-wire))
	  (sum (make-wire))
	  (carry (make-wire))
	  carries sums)
      (setf *agenda* (make-agenda 0))
      ;;(probe sum "sum")
      ;;(probe carry "carry")
      (accumulate-probe carries carry)
      (accumulate-probe sums sum)
      (half-adder in1 in2 sum carry or-delay and-delay inv-delay)
      (set-signal in1 1)
      (propagate)
      (set-signal in2 1)
      (propagate)
      (values (nreverse sums) (nreverse carries)))
  ((0 . 0) (3) (8 . T) (16))
  ((0 . 0) (3) (11 . T)))

(deftest test-clock
  (let ((clock-out (make-wire))
	clock-trans)
    (setf *agenda* (make-agenda 0))
    (accumulate-probe clock-trans clock-out)
    (make-clock clock-out 5 :rising 2 3)
    (propagate-until 25)
    (nreverse clock-trans))
  ((0 . 0) (5 . 1) (7 . 0) (10 . 1) (12 . 0) (15 . 1) 
   (17 . 0) (20 . 1) (22 . 0) (25 . 1) (27 . 0)))
  

      
	    
	  