;;;
;;; queue.lisp
;;;
;;; CLOS class & method to support queues.
;;;
;;; Support for circuit-sim.lisp
;;;
;;; $Revision: 1.2 $
;;;
;;; $Log: queue.lisp,v $
;;; Revision 1.2  2007/02/11 18:50:06  jao
;;; In package "CIRCUIT"
;;;
;;; Revision 1.1  2007/02/11 00:40:28  jao
;;; Initial revision
;;;
;;;
;;;

(in-package "CIRCUIT")

;;;
;;; QUEUE: insertions at rear, deletions at front
;;;

(defclass queue ()
  ((front-ptr :accessor front-ptr :initarg :front-ptr)
   (rear-ptr :accessor rear-ptr :initarg :rear-ptr)))

(defun make-queue ()
  (make-instance 'queue :front-ptr nil :rear-ptr nil))

(defmethod empty ((q queue))
  (null (front-ptr q)))

(defmethod insert ((q queue) item)
  (let ((new-pair (cons item nil)))
    (if (empty q)
        (progn
          (setf (front-ptr q) new-pair)
          (setf (rear-ptr q) new-pair)
          q)
        (progn
          (setf (cdr (rear-ptr q)) new-pair)
          (setf (rear-ptr q) new-pair)
          q))))

(defmethod front ((q queue))
  (if (empty q)
      (error "Queue empty")
      (car (front-ptr q))))

(defmethod rear ((q queue))
  (if (empty q)
      (error "Queue empty")
      (car (rear-ptr q))))

(defmethod delete-front ((q queue))
  (if (empty q)
      (error "Queue empty")
      (let ((f (front q)))
        (setf (front-ptr q) (cdr (front-ptr q)))
        (values q f))))
